property spriteNum, mySprite, myMember, myStandardMember, myRollovermember, myMouseDownMember, myDisabledMember, myMouseDownSound, myMouseUpSound, myActiveFlag, myXRayFlag, myMessageType, myMessage, theMouseWasUp, myMouseDown, myRollover, mySubscribersList

on getBehaviorDescription me
  return "PUSH BUTTON" & RETURN & RETURN & "This behavior sets the member of a sprite depending on the state of the mouse (elsewhere, rollover, mouseDown, mouseUp)." & RETURN & RETURN & "This creates a button which can either initiate actions in other sprites, or provide visual feedback for other behaviors attached to the same sprite." & RETURN & RETURN & "The behavior can be enabled or disabled, using a #PushButton_ToggleActive call to the behavior or the sprite." & RETURN & RETURN & "Two messaging systems are provided:" & RETURN & "1) A custom message can sent whenever the Push Button behavior is activated. " & "This message can be sent to a Movie Script handler, all sprites, or the actorList. " & "The message can also be suppressed." & RETURN & RETURN & "2) Objects can 'subscribe' to the behavior in order to receive PushButton_Activated, _Enabled and _Disabled messages. " & "A two-way messaging system allows for cleaning up object references before an object is destroyed." & RETURN & RETURN & "The behavior can be set to consider that all sprites in higher channels either block or let through all mouse events. " & "If mouse events are allowed to pass, you can place blended sprites above the button to change its color. " & "If mouse events are blocked, such translucent sprites provide an alternative method for disabling the button." & RETURN & RETURN & "PERMITTED MEMBER TYPES" & RETURN & "[#bitmap, #filmLoop, #flash, #movie, #picture, #quickTimeMedia, #shape, #vectorShape]" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Standard member (when mouse is elsewhere)" & RETURN & "* Rollover member" & RETURN & "* MouseDown member" & RETURN & "* Disabled member" & RETURN & RETURN & "Optional parameters:" & RETURN & "* MouseDown sound" & RETURN & "* MouseUp sound" & RETURN & "If members are placed consecutively in the cast in this order then default values can be used to create the button." & RETURN & RETURN & "* Do sprites above the button allow mouse events through?" & RETURN & "* Type of message sent on mouseUp: do | sendAllSprites | call the actorList | no action" & RETURN & "* Custom Message sent on mouseUp" & RETURN & RETURN & "NOTES:" & RETURN & "If you use 'do', be sure that you have a handler in a Movie Script that corresponds to the message sent." & RETURN & RETURN & "If you indicate 'no action' then this behavior will simply deal with the different states of the button. " & "You can still execute an action on mouseUp by one of two means:" & RETURN & "1) Add a behavior with a mouseUp handler to the same sprite (for example, the 'Jump Back Button' behavior" & RETURN & "2) Subscribe an object to the current behavior. " & " The object will then receive your Custom Message directly."
end

on getBehaviorTooltip me
  return "Use with graphic members." & RETURN & RETURN & "Swaps the member of the sprite according to the state of the mouse. " & "You can use this dynamic button behavior to play a brief sound on mouseDown and/or mouseUp, send out a custom message of your choice, and trigger actions of other sprites. " & "You can also use it to provide visual feedback for other behaviors on the same sprite (for example, Jump to Marker Button). " & "This behavior can also interact with custom objects."
end

on beginSprite me
  Initialize(me)
end

on exitFrame me
  if myActiveFlag then
    CheckForRollover(me)
  end if
end

on endSprite me
  call(#PushButton_ClosingDown, mySubscribersList, me, spriteNum, myMessage)
  mySubscribersList.deleteAll()
end

on Initialize me
  mySprite = sprite(me.spriteNum)
  myMember = mySprite.member
  myMouseDownSound = member("click")
  repeat while the last char in myMessage = SPACE
    delete char -30000 of myMessage
  end repeat
  if not ["do", "no action"].getPos(myMessageType) then
    if myMessage contains SPACE then
      ErrorAlert(me, #spaceInMessage, myMessage)
    else
      myMessage = symbol(myMessage)
    end if
  end if
  myActiveFlag = myActiveFlag = "Active"
  myXRayFlag = myXRayFlag = "let all mouse events through"
  myStandardMember = value(myStandardMember)
  myRollovermember = value(myRollovermember)
  myMouseDownMember = value(myMouseDownMember)
  myDisabledMember = value(myDisabledMember)
  myMouseDownSound = value(myMouseDownSound)
  myMouseUpSound = value(myMouseUpSound)
  if myActiveFlag then
    myMember = myStandardMember
    mySprite.member = myMember
  else
    myMember = myDisabledMember
    mySprite.member = myMember
  end if
  mySubscribersList = []
  sendAllSprites(#PushButton_OpenForBusiness, mySubscribersList, me, spriteNum, myMessage)
end

on CheckForRollover me
  mouseOverMe = rollover(spriteNum)
  if mouseOverMe then
    if not myXRayFlag then
      mouseOverMe = the mouseMember = myMember
    end if
  end if
  if myRollover = mouseOverMe then
    if theMouseWasUp = the mouseUp then
      exit
    else
      theMouseWasUp = the mouseUp
      if mouseOverMe then
        if the mouseUp then
          if myMouseDown then
            Activate(me)
          else
            myMember = myRollovermember
            mySprite.member = myMember
          end if
        else
          clickOn(me)
        end if
      else
        if the mouseUp then
          if myMouseDown then
            Disactivate(me)
          end if
        end if
      end if
    end if
  else
    myRollover = mouseOverMe
    if myMouseDown then
      if myRollover then
        myMember = myMouseDownMember
        mySprite.member = myMember
      else
        myMember = myStandardMember
        mySprite.member = myMember
      end if
    else
      if not (the mouseDown) and myRollover then
        myMember = myRollovermember
        mySprite.member = myMember
      else
        myMember = myStandardMember
        mySprite.member = myMember
      end if
    end if
  end if
end

on clickOn me
  myMouseDown = 1
  myMember = myMouseDownMember
  mySprite.member = myMember
  if not voidp(myMouseDownSound) and (myMouseDownSound <> #none) then
    puppetSound(myMouseDownSound)
  end if
  updateStage()
end

on Activate me
  myMouseDown = 0
  theMouseWasUp = 0
  myMember = myStandardMember
  mySprite.member = myMember
  if not voidp(myMouseUpSound) and (myMouseUpSound <> #none) then
    puppetSound(myMouseUpSound)
  end if
  updateStage()
  case myMessageType of
    "do":
      do(myMessage)
    "sendAllSprites":
      sendAllSprites(myMessage, me, spriteNum)
    "call the actorList":
      call(myMessage, the actorList, me, spriteNum)
  end case
  call(#PushButton_Activated, mySubscribersList, me, spriteNum, myMessage)
end

on Disactivate me
  myMouseDown = 0
  myMember = myStandardMember
  mySprite.member = myMember
end

on PushButton_ToggleActive me, trueOrFalse
  if voidp(trueOrFalse) then
    myActiveFlag = not myActiveFlag
  else
    case ilk(trueOrFalse) of
      #integer:
        myActiveFlag = trueOrFalse <> 0
      otherwise:
        return #invalidTypeError
    end case
  end if
  if myActiveFlag then
    myMember = myStandardMember
    mySprite.member = myMember
    call(#PushButton_Enabled, mySubscribersList, me, spriteNum, myMessage)
  else
    myMember = myDisabledMember
    mySprite.member = myMember
    call(#PushButton_Disabled, mySubscribersList, me, spriteNum, myMessage)
  end if
end

on PushButton_GetReference me, theList
  case ilk(theList) of
    #list:
      theList.append(me)
    #propList:
      theList.addProp(me.spriteNum, me)
    otherwise:
      return me
  end case
  return theList
end

on PushButton_Subscribe me, callingBehavior, theMessage
  if not voidp(theMessage) then
    if theMessage <> myMessage then
      exit
    end if
  end if
  if mySubscribersList.getPos(callingBehavior) then
    exit
  else
    if objectp(callingBehavior) then
      mySubscribersList.append(callingBehavior)
      return me
    end if
  end if
end

on PushButton_Unsubscribe me, callingBehavior
  mySubscribersList.deleteOne(callingBehavior)
end

on substituteStrings me, parentString, childStringList
  i = childStringList.count()
  repeat while i
    tempString = EMPTY
    dummyString = childStringList.getPropAt(i)
    replacement = childStringList[i]
    lengthAdjust = dummyString.char.count - 1
    repeat while 1
      position = offset(dummyString, parentString)
      if not position then
        parentString = tempString & parentString
        exit repeat
        next repeat
      end if
      if position <> 1 then
        tempString = tempString & parentString.char[1..position - 1]
      end if
      tempString = tempString & replacement
      delete parentString.char[1..position + lengthAdjust]
    end repeat
    i = i - 1
  end repeat
  return parentString
end

on ErrorAlert me, theError, data
  case theError of
    #spaceInMessage:
      if the runMode = "Author" then
        behaviorName = string(me)
        delete word 1 of behaviorName
        delete char -30001 of behaviorName
        delete char -30001 of behaviorName
        terror1 = "BEHAVIOR ERROR: Frame ^0, Sprite ^1"
        terror1 = substituteStrings(me, terror1, ["^0": the frame, "^1": the currentSpriteNum])
        terror2 = "Behavior ^0"
        terror2 = substituteStrings(me, terror2, ["^0": behaviorName])
        terror3 = "This message includes spaces: ^0"
        terror3 = substituteStrings(me, terror3, ["^0": QUOTE & data & QUOTE])
        terror4 = "Only the first word will be used in sendAllSprite calls."
        alert(terror1 & RETURN & RETURN & terror2 & RETURN & RETURN & terror3 & RETURN & RETURN & terror4)
      end if
  end case
end

on isOKToAttach me, aSpriteType, aSpriteNum
  tisok = 0
  if aSpriteType = #graphic then
    if PermittedMemberTypes().getOne(sprite(aSpriteNum).member.type) > 0 then
      tisok = 1
    end if
  end if
  return tisok
end

on getPropertyDescriptionList me
  theMember = sprite(the currentSpriteNum).member
  memberType = theMember.type
  permittedTypes = PermittedMemberTypes(me)
  theMemberNum = theMember.number
  suitableMembersList = GetSuitableMembers(me, permittedTypes)
  descriptionList = [#myStandardMember: [#comment: "- GRAPHICS - Standard member for sprite", #format: #member, #range: suitableMembersList, #default: theMember], #myRollovermember: [#comment: "Rollover member", #format: #member, #range: suitableMembersList, #default: member(theMemberNum - 1)], #myMouseDownMember: [#comment: "MouseDown member", #format: #member, #range: suitableMembersList, #default: member(theMemberNum - 2)], #myDisabledMember: [#comment: "Disabled member", #format: #member, #range: suitableMembersList, #default: member(theMemberNum - 1)]]
  soundsAvailable = GetSuitableMembers(me, [#sound])
  if soundsAvailable.count() then
    soundsAvailable.addAt(1, #none)
    descriptionList.addProp(#myMouseDownSound, [#comment: "- SOUNDS - Sound to play on mouseDown", #format: #sound, #range: soundsAvailable, #default: member("click")])
    descriptionList.addProp(#myMouseUpSound, [#comment: "Sound to play on mouseUp", #format: #sound, #range: soundsAvailable, #default: member(theMemberNum + 5)])
  end if
  descriptionList.addProp(#myActiveFlag, [#comment: "- INTERACTION - Button is initially", #format: #string, #range: ["Active", "Inactive"], #default: "Active"])
  descriptionList.addProp(#myXRayFlag, [#comment: "Sprites which cover the button", #format: #string, #range: ["block all mouse events", "let all mouse events through"], #default: 0])
  descriptionList.addProp(#myMessageType, [#comment: "Action on mouseUp", #format: #string, #range: ["do", "sendAllSprites", "call the actorList", "no action"], #default: "sendAllSprites"])
  descriptionList.addProp(#myMessage, [#comment: EMPTY, #format: #string, #default: "YourCustomMessage"])
  return descriptionList
end

on GetSuitableMembers me, permittedTypes
  cursor(4)
  suitableMembersList = []
  maxCastLib = the number of castLibs
  repeat with theCastLib = 1 to maxCastLib
    maxMember = the number of castMembers of castLib theCastLib
    repeat with memberNumber = 1 to maxMember
      theMember = member(memberNumber, theCastLib)
      if permittedTypes.getPos(theMember.type) then
        if theMember.name = EMPTY then
          suitableMembersList.append(theMember)
          next repeat
        end if
        suitableMembersList.append(theMember.name)
      end if
    end repeat
  end repeat
  cursor(-1)
  return suitableMembersList
end

on PermittedMemberTypes me
  return [#bitmap, #filmLoop, #flash, #movie, #picture, #quickTimeMedia, #shape, #vectorShape]
end
